#!/usr/bin/env python
'''
This script will remove roughly half of the files in a given directory in a
somewhat random fashion.  If no directories are given, it will remove roughly
half of the files in the present working directory.  This is a dangerous script,
obviously.  Use with caution.

Program:    halver.py
Version:    0.8
License:    GPL 3 (see LICENSE)
Author:     James Hendrie (hendrie dot james at gmail dot com)
Last edit:  2012-07-29
'''

import os, sys


def exit_program():
    '''
    Exits the program.
    '''
    print("\n\nExiting.\n")
    sys.exit()

def get_input(queryString):
    '''
    A function that lets this program work regardless of which verison of
    Python the user is running.
    '''
    ## Exists to check if Python version 2 or 3 is being used.
    versionMajor = sys.version[0]
    try:
        if int(versionMajor) == 2:
            userInput = raw_input(queryString)
        elif int(versionMajor) > 2:
            userInput = input(queryString)
    except KeyboardInterrupt:
        exit_program()

    return userInput


def get_confirmation(options, dirsPassed):
    '''
    Get confirmation from the user, assuming the 'safe' option is set to 1.
    '''
    noteStr = """
    Note:  You are about to delete roughly half of the contents in all of
           the following directories:
    """
    print(noteStr)
    for d in dirsPassed:
        print(d)

    print("\n")

    queryString = "Do you wish to proceed?  [Y/N]"
    uI = get_input(queryString)
    
    if uI == 'n' or uI == 'N' or uI == 'no' or uI == 'NO':
        print("\n\nVery well.  Exiting.\n")
        sys.exit()
    elif uI == 'y' or uI == 'Y' or uI == 'yes' or uI == 'YES':
        options['safe'] = 0
    else:
        print("\n\n[Y]es or [N]o, please.")

    return options



def remover(options, dirsPassed):
    '''
    Remove half of the files from the passed directories.
    '''
    for d in dirsPassed:
        dirName = os.path.abspath(d)
        dirList = os.listdir(dirName)
        x = 0
        os.chdir(dirName)
        if options['verbose'] == 1:
            print("\nChanging to directory", dirName)
        for i in dirList:
            x += 1
            if x == 2:
                if options['verbose'] == 1:
                    print("Removing ", i)
                os.remove(i)
                x = 0

def get_options():
    '''
    Get options from the user.  This is a fast but not particularly beautiful
    way of impletementing this, I know.  You can also just set default options
    here, if you want.  1 = True, 0 = False.  Verbosity and Safe are probably
    the two options you may wish to look at, though by default verbosity is off
    while safe is on.
    '''
    options = {}

    ## Defaults
    options['verbose'] = 0  ##  Verbose; print what you're removing line by line
    options['help'] = 0     ##  Print help
    options['version'] = 0  ##  Print version
    options['safe'] = 1     ##  Use the confirmation before removing stuff

    for i in sys.argv:
        if i == '-v':
            options['verbose'] = 1
        elif i == '--verbose':
            options['verbose'] = 1
        elif i == '-h':
            options['help'] = 1
        elif i == '--help':
            options['help'] = 1
        elif i == '--version':
            options['version'] = 1
        elif i == '-y':
            options['safe'] = 0
        elif i == '--yes':
            options['safe'] = 0
        elif i == '-yv':
            options['safe'] = 0
            options['verbose'] = 1
        elif i == '-vy':
            options['safe'] = 0
            options['verbose'] = 1

    return options

def get_dirs():
    '''
    This goes through the arguments passed and determines whether or not they're
    directories.  If no valid directories are found, the present working
    directory will be used.
    '''
    dirsGiven = []
    dirsPassed = []
    for i in sys.argv:
        ##  See if they're trying to use options
        if '-' in i or '--' in i:
            pass
        else:
            dirsGiven.append(i)

    for i in dirsGiven:
        if os.path.isdir(i):
            ## Determine if an arg is actually a directory
            dirsPassed.append(os.path.abspath(i))

    if len(dirsPassed) < 1:
        ## If no directories are given yet, use present working directory
        curDir = os.path.abspath(os.getcwd())
        if os.path.isdir(curDir):
            dirsPassed.append(curDir)

    if len(dirsPassed) < 1:
        ##  If all else fails, whine and quit.
        print("Error:  No viable directories passed.")
        sys.exit()

    return dirsPassed


def print_help():
    '''
    This prints the help.
    '''
    helpStr = """

    This program will go through a list of given directories, removing (roughly)
    half of the files from each one at random.  If no directories are given, the
    program will run through the current working directory instead.

    Why would you want to do this?  If you've accumulated a bunch of stuff you
    don't cherish too much, but still want to keep some of it (say, pornography
    for example), this program will keep you from wasting hours trying to
    organize all of it.

    Options:
        -h or --help:       Print this help
        --version:          Print verion
        -v or --verbose:    Turn verbosity on
        -y or --yes:        Turn 'safe mode' off (auto-confirm)

        You can combine verbosity and auto-confirm if you like.  '-yv' will
        be verbose while skipping confirmation for file removal.

        """
    print(helpStr)



def print_version():
    '''
    This prints the version number and author info.
    '''
    versionStr = """

    Program:    halver.py
    Version:    0.8
    Author:     James Hendrie (hendrie dot james at gmail dot com)
    Last edit:  2012-07-29

    """
    print(versionStr)

def main():
    '''
    Alright, here we go!!!!  YEEAAAHH
    '''
    ##  Get options
    options = get_options()

    if options['help'] == 1:
        ##  print help
        print_help()
        sys.exit()

    if options['version'] == 1:
        ##  print version
        print_version()
        sys.exit()

    ##  Get the directories to be used
    dirsPassed = get_dirs()

    if options['safe'] == 0:
        try:
            remover(options, dirsPassed)
        except KeyboardInterrupt:
            exit_program()

    elif options['safe'] == 1:
        ##  Safety first!
        while options['safe'] == 1:
            options = get_confirmation(options, dirsPassed)
        try:
            remover(options, dirsPassed)
        except KeyboardInterrupt:
            exit_program()

if __name__ == "__main__":
    main()
